//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_LEADERVEHICLE_H
#define PROTOCOL_LEADERVEHICLE_H


#include <mutex>
#include <unordered_map>
#include <bits/unordered_map.h>
#include "Vehicle.h"
#include "NetworkServiceServer.h"
#include "PlatoonCreateNetworkService.h"

namespace protocol
{

class LeaderVehicle : public Vehicle
{
public:
    LeaderVehicle(
        boost::asio::io_service & ioService,
        VehicleId vehicleId)
        : Vehicle(ioService, vehicleId, Role::LEADER)
    {}

    std::size_t getNumFollowers()
    {
        std::lock_guard<std::mutex> lock(followersMutex);
        return followers.size();
    }

protected:
    void doOnCreatePlatoon() override;

    void doCreatePlatoon() override;

    void doOnRunningPlatoon() override;

    void doRunningPlatoon() override;

    void doOnLeavingPlatoon() override;

    void doLeavingPlatoon() override;

private:
    std::unordered_map<VehicleId, VehicleEndpoint> followers{2};
    std::mutex followersMutex;
    std::vector<VehicleEndpoint>::iterator vehicleEndpointIter;

    void makePlatoonCreateRequest(VehicleEndpoint vehicleEndpoint);

    PlatoonCreateNetworkMessage makePlatoonCreateResponse(
        const tcp::endpoint & clientEndpoint, const PlatoonCreateNetworkMessage & request);
};

}


#endif //PROTOCOL_LEADERVEHICLE_H
