#include <iostream>
#include "Protocol.h"
#include "LeaderVehicle.h"
#include "FollowerVehicle.h"
#include "Networking.h"

#define NETWORKING_TEST

#if defined(NETWORKING_TEST)

#include "NetworkingTest.h"

int main(int argc, char ** argv)
{
    std::cout << "\ntestService\n\n";
    networking::test::testServices();
    std::cout <<"\ntestTcpClientTimeout\n\n";
    networking::test::testTcpClientTimeout();
    std::cout <<"\ntestMultipleConnections\n\n";
    networking::test::testMultipleConnections();
    std::cout <<"\ntestTcpServerChangePort\n\n";
    networking::test::testTcpServerChangePort();
    return 0;
}

#elif defined(PROTOCOL_TEST)

#include "ProtocolTest.h"

int main(int argc, char ** argv)
{
    std::cout << "testStop\n\n";
    protocol::test::testStop();

    return 0;
}

#else

int main(int argc, char ** argv)
{
    networking::Networking net;
    auto & ioService = net.getIoService();

    protocol::LeaderVehicle leader(ioService, 1);
    protocol::FollowerVehicle follower1(ioService, 2);
    protocol::FollowerVehicle follower2(ioService, 3);

    leader.createPlatoon();
    follower1.createPlatoon();
    follower2.createPlatoon();

    while(!leader.isIdle() || !follower1.isIdle() || !follower2.isIdle())
    {
        leader.run();
        follower1.run();
        follower2.run();

        if (leader.isPlatoonRunning() && leader.getNumFollowers() == 2)
        {
            // just for printing
            leader.run();
            follower1.run();
            follower2.run();

            leader.leavePlatoon();
            follower1.leavePlatoon();
            follower2.leavePlatoon();
        }
    }

    return 0;
}

#endif