//
// Created by philipp on 06.12.17.
//

#ifndef PROTOCOL_NETWORKING_H
#define PROTOCOL_NETWORKING_H

#include <boost/asio/io_service.hpp>
#include <thread>
#include "NetworkServiceClient.h"

namespace networking
{

class Networking
{
public:
    Networking()
    {
        work = std::make_unique<boost::asio::io_service::work>(ioService);
        thread = std::thread(
            [this]()
            { ioService.run(); });
    }

    ~Networking()
    {
        work.reset();
        ioService.stop();
        thread.join();
    }

    Networking(const Networking & other) = delete;

    Networking & operator=(const Networking & other) = delete;

    Networking(Networking && other) = delete;

    Networking & operator=(Networking && other) = delete;

    boost::asio::io_service & getIoService()
    { return ioService; }

private:
    boost::asio::io_service ioService;
    std::unique_ptr<boost::asio::io_service::work> work;
    std::thread thread;
};


namespace service
{

namespace tcp
{
template<class Service>
using Server = typename networking::service::Server<Service, boost::asio::ip::tcp>;

template<class Service>
using Client = typename networking::service::Client<Service, boost::asio::ip::tcp>;
}

namespace udp
{
template<class Service>
using Server = typename networking::service::Server<Service, boost::asio::ip::udp>;

template<class Service>
using Client = typename networking::service::Client<Service, boost::asio::ip::udp>;
}

}

}

#endif //PROTOCOL_NETWORKING_H
