//
// Created by philipp on 02.12.17.
//

#include "NetworkingTest.h"
#include <boost/asio/ip/tcp.hpp>
#include <iostream>
#include "NetworkServiceServer.h"
#include "PlatoonCreateNetworkService.h"
#include "NetworkServiceClient.h"
#include "Networking.h"

using boost::asio::ip::tcp;

void networking::test::testServices()
{
    using namespace protocol;
    Networking net;
    auto & ioService = net.getIoService();

    service::tcp::Server<PlatoonCreateNetworkService> server{ioService};
    server.advertiseService(
        "127.0.0.1", 10001,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            std::cout << "Request from " << (int) requestMessage.getVehicleId() << " with type: "
                      << (int) requestMessage.getMessageType() << "\n";
            return PlatoonCreateNetworkMessage::acceptResponse(1, 42);
        });

    sleep(3);

    service::tcp::Client<PlatoonCreateNetworkService> client{ioService};
    for (int i = 0; i < 5; i++)
    {
        auto response = client.call("127.0.0.1",
                                    10001,
                                    PlatoonCreateNetworkMessage::followerRequest(2),
                                    boost::posix_time::seconds(5));
        std::cout << "Response from " << (int) response.getVehicleId() << " with type: "
                  << (int) response.getMessageType()
                  << " and platoonId: " << (int) response.getPlatoonId() << "\n";
    }
}

void ::networking::test::testTcpClientTimeout()
{
    using namespace protocol;
    Networking net;
    auto & ioService = net.getIoService();

    const auto timeout = boost::posix_time::seconds(3);

    service::tcp::Server<PlatoonCreateNetworkService> server{ioService};
    server.advertiseService(
        "127.0.0.1", 10001,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            // Just sleep for 3 seconds.
            sleep(5);
            return PlatoonCreateNetworkMessage::acceptResponse(1, 42);
        });

    sleep(1);

    service::tcp::Client<PlatoonCreateNetworkService> client{ioService};
    auto startTime = boost::posix_time::microsec_clock::local_time();
    try
    {
        client.call("127.0.0.1",
                    10001,
                    PlatoonCreateNetworkMessage::followerRequest(2),
                    timeout);
        std::cout << "FAILED!";
    }
    catch (const std::exception & e)
    {
        auto nowTime = boost::posix_time::microsec_clock::local_time();
        auto timeSpend = nowTime - startTime;
        if (timeSpend.seconds() >= 2)
            std::cout << "SUCCESS!\n";
    }
}

void ::networking::test::testMultipleConnections()
{
    using namespace protocol;
    Networking net;
    auto & ioService = net.getIoService();

    service::tcp::Server<PlatoonCreateNetworkService> server1{ioService};
    service::tcp::Server<PlatoonCreateNetworkService> server2{ioService};

    server1.advertiseService(
        "127.0.0.1", 10001,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            return PlatoonCreateNetworkMessage::acceptResponse(1, 42);
        });

    server2.advertiseService(
        "127.0.0.1", 10002,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            return PlatoonCreateNetworkMessage::acceptResponse(2, 43);
        });

    sleep(1);

    service::tcp::Client<PlatoonCreateNetworkService> client{ioService};

    auto response = client.call("127.0.0.1",
                                 10001,
                                 PlatoonCreateNetworkMessage::followerRequest(1),
                                 boost::posix_time::seconds(5));
    std::cout << "Response from " << response.getVehicleId() << std::endl;

    response = client.call("127.0.0.1",
                                 10002,
                                 PlatoonCreateNetworkMessage::followerRequest(2),
                                 boost::posix_time::seconds(5));
    std::cout << "Response from " << response.getVehicleId() << std::endl;
}

void ::networking::test::testTcpServerChangePort()
{
    using namespace protocol;
    Networking net;
    auto & ioService = net.getIoService();

    service::tcp::Client<PlatoonCreateNetworkService> client{ioService};

    service::tcp::Server<PlatoonCreateNetworkService> server{ioService};
    server.advertiseService(
        "127.0.0.1", 10001,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            return PlatoonCreateNetworkMessage::acceptResponse(1, 42);
        });

    server.advertiseService(
        "127.0.0.1", 10002,
        [](const tcp::endpoint & clientEndpoint,
           const PlatoonCreateNetworkMessage & requestMessage) -> PlatoonCreateNetworkMessage
        {
            return PlatoonCreateNetworkMessage::acceptResponse(1, 42);
        });

    sleep(1);

    auto response = client.call("127.0.0.1",
                                10002,
                                PlatoonCreateNetworkMessage::followerRequest(1),
                                boost::posix_time::seconds(5));

    if (response.getMessageType() == messageTypes::ACCEPT_RESPONSE)
        std::cout << "SUCCESS!\n";
}

