//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_VEHICLE_H
#define PROTOCOL_VEHICLE_H

#include <thread>
#include <boost/asio/io_service.hpp>
#include <boost/asio.hpp>
#include "VehicleEndpoint.h"
#include "NetworkServiceServer.h"
#include "PlatoonCreateNetworkService.h"
#include "Networking.h"

namespace protocol
{
class Vehicle
{
public:
    enum class Role
    {
        LEADER, FOLLOWER
    };

    virtual ~Vehicle()
    {};

    bool run();

    bool isIdle();

    bool isPlatoonRunning();

    bool createPlatoon();

    bool leavePlatoon();

    Role getRole() const
    { return role; }

protected:
    Vehicle(
        boost::asio::io_service & ioService,
        VehicleId vehicleId,
        Role role)
        : ioService(ioService)
          , vehicleId(vehicleId)
          , role(role)
          , platoonCreateServiceServer(ioService)
          , platoonCreateServiceClient(ioService)
    {}

    enum class State
    {
        IDLE,
        ON_CREATING_PLATOON,
        CREATING_PLATOON,
        ON_RUNNING_PLATOON,
        RUNNING_PLATOON,
        ON_LEAVING_PLATOON,
        LEAVING_PLATOON
    };

    VehicleId vehicleId;
    PlatoonId platoonId;
    State state{State::IDLE};
    boost::asio::io_service & ioService;
    networking::service::tcp::Server<PlatoonCreateNetworkService> platoonCreateServiceServer;
    networking::service::tcp::Client<PlatoonCreateNetworkService> platoonCreateServiceClient;
    std::vector<VehicleEndpoint> availableVehicleEndpoints;

    virtual void doOnCreatePlatoon() = 0;

    virtual void doCreatePlatoon() = 0;

    virtual void doOnRunningPlatoon() = 0;

    virtual void doRunningPlatoon() = 0;

    virtual void doOnLeavingPlatoon() = 0;

    virtual void doLeavingPlatoon() = 0;

    void scanAvailableVehicles();

private:
    Role role;
};
}


#endif //PROTOCOL_VEHICLE_H
